==> open-xchange-documents-backend_7.8.2.orig.tar.bz2: open-xchange-documents-backend-7.8.2/com.openexchange.office.odf/src/com/openexchange/office/odt/dom/RowHandler.java <==
/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2016 OX Software GmbH
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

/**
 * @author sven.jacobi@open-xchange.com
 */

package com.openexchange.office.odt.dom;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;

import com.openexchange.office.FilterException;
import com.openexchange.office.FilterException.ErrorCode;
import com.openexchange.office.odf.DLList;
import com.openexchange.office.odf.SaxContextHandler;

public class RowHandler extends SaxContextHandler {

	final Table table;
	final Row row;
	final int maxColumns;
	int repeated;

	public RowHandler(SaxContextHandler parentContext, Table table, Attributes attributes, boolean headerRow, int maxColumns, int maxRows, int maxCells)
    	throws SAXException {

		super(parentContext);

		this.table = table;
		this.row = new Row(attributes, headerRow);
		this.maxColumns = maxColumns;

		repeated = 1;
        try {
        	final String val = attributes.getValue("table:number-rows-repeated");
        	if(val!=null) {
        		repeated = Integer.valueOf(Integer.parseInt(val));
        	}
        } catch (NumberFormatException e) {
        }
        final DLList<Object> rows = table.getContent();
        if(rows.size() + repeated > maxRows) {
        	throw new FilterException("ODT Document exceeds row limit", ErrorCode.COMPLEXITY_TOO_HIGH);
        }
        if((rows.size() + repeated) * table.getColumns().size() > maxCells) {
        	throw new FilterException("ODT Document exceeds cell limit", ErrorCode.COMPLEXITY_TOO_HIGH);
        }
        rows.add(row);
	}

	@Override
	public SaxContextHandler startElement(Attributes attributes, String uri, String localName, String qName)
			throws SAXException {

		if(qName.equals("table:table-cell")) {
    		return new CellHandler(this, row, attributes, maxColumns);
    	}
		return this;
	}

	@Override
	public void endContext(String qName, String characters) {
		super.endContext(qName, characters);

		int gridSpan = 0;
		for(Object o:row.getContent()) {
			gridSpan += ((Cell)o).getColumnSpan();
		}
		if(table.getMaxRowGrid() < gridSpan) {
			table.setMaxRowGrid(gridSpan);
		}
		final DLList<Object> rows = table.getContent();
        while(repeated-->1) {
			rows.add(row.clone());
        }
	}
}

==> open-xchange-documents-backend_7.8.3.orig.tar.bz2: open-xchange-documents-backend-7.8.3/com.openexchange.office.odf/src/com/openexchange/office/odt/dom/RowHandler.java <==
/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2016 OX Software GmbH
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

/**
 * @author sven.jacobi@open-xchange.com
 */

package com.openexchange.office.odt.dom;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;

import com.openexchange.office.odf.DLList;
import com.openexchange.office.odf.SaxContextHandler;

public class RowHandler extends SaxContextHandler {

	final Table table;
	final Row row;
	final int maxColumns;
	int repeated;

	public RowHandler(SaxContextHandler parentContext, Table table, Attributes attributes, boolean headerRow, int maxColumns, int maxRows, int maxCells)
    	throws SAXException {

		super(parentContext);

		this.table = table;
		this.row = new Row(attributes, headerRow);
		this.maxColumns = maxColumns;

		repeated = 1;
        try {
        	final String val = attributes.getValue("table:number-rows-repeated");
        	if(val!=null) {
        		repeated = Integer.valueOf(Integer.parseInt(val));
        	}
        } catch (NumberFormatException e) {
        }
        final DLList<Object> rows = table.getContent();
        if((rows.size() + repeated) - 1 > maxRows) {
        	table.setTableSizeExceeded(true);
        }
        if(((rows.size() + repeated) - 1) * table.getColumns().size() > maxCells) {
        	table.setTableSizeExceeded(true);
        }
        rows.add(row);
	}

	@Override
	public SaxContextHandler startElement(Attributes attributes, String uri, String localName, String qName)
			throws SAXException {

		if(qName.equals("table:table-cell")) {
    		return new CellHandler(this, table, row, attributes, maxColumns);
    	}
		return this;
	}

	@Override
	public void endContext(String qName, String characters) {
		super.endContext(qName, characters);

		int gridSpan = 0;
		for(Object o:row.getContent()) {
			gridSpan += ((Cell)o).getColumnSpan();
		}
		if(table.getMaxRowGrid() < gridSpan) {
			table.setMaxRowGrid(gridSpan);
		}
		final DLList<Object> rows = table.getContent();
        while(repeated-->1) {
			rows.add(row.clone());
        }
	}
}
